package yrpg.slm.pc.plugin.operate;

import kd.bos.algo.DataSet;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.PreparePropertysEventArgs;
import kd.bos.entity.plugin.args.BeforeOperationArgs;
import kd.bos.orm.ORM;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.QueryServiceHelper;
import kd.sdk.plugin.Plugin;
import yrpg.slm.pc.common.GrantCommonRules;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

/**
 * renxing
 * 单据操作插件
 *   奖项保存时计算每个奖项的金额
 */
public class AwardDetailSaveOpPlugin extends AbstractOperationServicePlugIn implements Plugin {

    private DataSet grantDataSet = null;//工资发放单 该奖项数据
    private DataSet quarterGrantDataSet = null;//季度工资发放单 达成奖奖项数据
    private DataSet awardTypeDataSet = null;//所有奖项
    private DataSet projectScoreDataSet = null;//科研项目进度
    private DataSet nowBehaviorDataSet = null;//考核表

    private DataSet needQuarterDataSet =null;

    @Override
    public void onPreparePropertys(PreparePropertysEventArgs e) {
        super.onPreparePropertys(e);
        e.getFieldKeys().add("yrpg_srcid");
        e.getFieldKeys().add("yrpg_user");
        e.getFieldKeys().add("yrpg_period");
        e.getFieldKeys().add("yrpg_totalamount");
    }

    @Override
    public void beforeExecuteOperationTransaction(BeforeOperationArgs e) {
        int i = 0;
        for (DynamicObject obj : e.getDataEntities()) {

            String billKey = obj.getDataEntityType().getName();//获取当前单据的标识
            if(!billKey.contains("grant")){//
                break;//只有发放奖项
            }
            //源单标识
            String srcid = obj.getString("yrpg_srcid");
            //获取当前人员
            DynamicObject person = obj.getDynamicObject("yrpg_user");
            //获取当前期间
            DynamicObject period = obj.getDynamicObject("yrpg_period");

            if(i == 0) {//只有第一次加载
                this.getDataSet("yrpg_wages_grant",period,null);//将所有人的工资发放该奖项的数据源加载
                this.getDataSet("yrpg_pc_awardtype",period,null);//将奖项加载
                this.getDataSet("yrpg_behaviorscore",period,null);//加载考核表
            }

            //奖项类型 由奖项发放标识获取
            DataSet nowAwardDataSet = awardTypeDataSet.copy().filter("yrpg_grand = " + "'" +billKey+ "'" );
            DynamicObjectCollection awardTypeCollection = ORM.create().toPlainDynamicObjectCollection(nowAwardDataSet.copy());
            if(awardTypeCollection.size() == 0){
                //找不到奖项
                continue;
            }
            long awardPkid = awardTypeCollection.stream().findFirst().get().getLong("id");
            //奖项数据源 工资发放单获取到当前单据头人员、单据体奖项过滤之后的DataSet
            DataSet nowGrantDataSet = grantDataSet.copy().filter("yrpg_user = " + person.getPkValue() + " and awardtype = " + awardPkid);

            //TODO  业绩奖通用计算方式
            if("yrpg_grant_achievement".equals(billKey) || "yrpg_grant_area".equals(billKey)) {
                //this.GrantAchievementGet(person,period,prizeEntry);
                //每个人科研进度合计
                DataSet personProgressDataSet = nowBehaviorDataSet.copy().filter("user = "+ person.getPkValue());
                BigDecimal grantAmount = GrantCommonRules.grantAchievementRules(personProgressDataSet,nowGrantDataSet);
                obj.set("yrpg_totalamount",grantAmount);//奖金计算赋值
            }

            //TODO 达成奖-基层通用计算方式
            else if ("yrpg_grant_reachbase".equals(billKey) || "yrpg_grant_reachbaseoper".equals(billKey)) {
                //达成奖需要判断是否为本季度的最后一个月
                //if(isLastMonth(period)){

                //获取工资计算奖项标识
                String yrpgCompute = awardTypeCollection.stream().findFirst().get().getString("yrpg_compute");
                if(i == 0){
                    getDataSet("quarterGrant",period,awardPkid);//达成奖
                    getQuarterGrantDataSet(period,yrpgCompute);
                }

                BigDecimal grantAmount = GrantCommonRules.grantBehaviorRule(person,needQuarterDataSet.copy());
                obj.set("yrpg_totalamount", grantAmount);//奖金计算赋值
                //}
            }

            //TODO 普通奖项通用计算方式
            else {
                //普通奖项金额计算
                BigDecimal grantAmount = GrantCommonRules.grantNomalRule(nowGrantDataSet);
                obj.set("yrpg_totalamount",grantAmount);//奖金计算赋值
            }

            i++;//之后不加载所有来源数据
        }
    }

    public void getDataSet(String billMark,DynamicObject period,Long awardPkId){
        QFilter periodQf = new QFilter("yrpg_period", QCP.equals,period.getPkValue());//期间-期间年度查询条件：
        List<Integer> alllMonth = getAllMonth(period);
        QFilter allMonthQf = new QFilter("yrpg_period.periodnumber", QCP.in,alllMonth.toArray());//期间-期间年度查询条件
        QFilter awardQf = new QFilter("yrpg_sourceentryentity1.yrpg_awardtype", QCP.equals,awardPkId);//

        switch (billMark){
            case  "yrpg_wages_grant"://获取发放单
                //按照期间查询所有人在该会计期间内的工资发放单 该奖项数据
                grantDataSet = QueryServiceHelper.queryDataSet(this.getClass().toString(), billMark,
                        "yrpg_user,yrpg_period,yrpg_sourceentryentity1.yrpg_awardtype as awardtype,yrpg_sourceentryentity1.yrpg_beramount_in as beramountin", new QFilter[]{periodQf}, null);
                break;
            case  "quarterGrant"://获取当前季度这个奖项的发放数据
                quarterGrantDataSet = QueryServiceHelper.queryDataSet(this.getClass().toString(), "yrpg_wages_grant",
                        "yrpg_user,yrpg_period,yrpg_sourceentryentity1.yrpg_awardtype as awardtype,yrpg_sourceentryentity1.yrpg_beramount_in as beramountin", new QFilter[]{awardQf,allMonthQf}, null);
                break;
            case "yrpg_pc_awardtype"://奖项 id、计算标识、发放标识、
                awardTypeDataSet = QueryServiceHelper.queryDataSet(this.getClass().toString(), billMark,
                        "id,yrpg_compute,yrpg_grand", null, null);
                break;
            case "yrpg_sc_projectscore"://科研项目进度表
                projectScoreDataSet = QueryServiceHelper.queryDataSet(this.getClass().toString(), billMark,
                        "id,yrpg_examine_period as period,yrpg_point_list.yrpg_scienceproject as scienceProject,yrpg_point_list.yrpg_username as user,yrpg_point_list.yrpg_now_progress as nowProgress" +
                                ",yrpg_point_list.yrpg_last_progress as lastProgress,yrpg_point_list.yrpg_realmoney as realMoney"
                        ,new QFilter[]{periodQf}, null);
                break;
            case "yrpg_behaviorscore"://考核表
                QFilter nowPeriodQf = new QFilter("yrpg_check_date", QCP.equals,period.getPkValue());
                nowBehaviorDataSet = QueryServiceHelper.queryDataSet(this.getClass().toString(), billMark,
                        "id,yrpg_check_date,yrpg_assessor.yrpg_check_userid as user,yrpg_assessor.yrpg_compliancetotal as compliance"
                        ,new QFilter[]{nowPeriodQf}, null);
                break;
            default:
                break;
        }


    }

    public void getQuarterGrantDataSet(DynamicObject period,String yrpgCompute){
        List<Integer> alllMonth = getAllMonth(period);
        QFilter allMonthQf = new QFilter("yrpg_period.periodnumber", QCP.in,alllMonth.toArray());//期间-期间年度查询条件
        DataSet awardData =
                QueryServiceHelper.queryDataSet(this.getClass().toString(),yrpgCompute,"id,yrpg_period,yrpg_user,yrpg_prize_entry.yrpg_complate_rate as rate", new QFilter[]{allMonthQf},null);

        needQuarterDataSet = quarterGrantDataSet.copy().leftJoin(awardData).on("yrpg_user","yrpg_user").select(new String[]{"yrpg_user","beramountin"},new String[]{"rate"}).finish();
    }


    public void grantAchievementGet(DynamicObject person,DynamicObject period,DynamicObjectCollection prizeEntry){
        //获取当前人员当期 科研打分明细
        DataSet nowScoreDataSet = projectScoreDataSet.where("yrpg_user = " + person.getPkValue() + "and period = " + period); //科研项目进度表
        DynamicObjectCollection scoreCollection = ORM.create().toPlainDynamicObjectCollection(nowScoreDataSet.copy());

        BigDecimal amount = new BigDecimal(0);//获取实发金额
        //存在当期人员的打分表
        if (scoreCollection.getRowCount() > 0) {
            for (DynamicObject scoreCol : scoreCollection) {
                //获取当前人员 当前科研项目 所有期间  打分
                DataSet allProjectScoreDataSet = projectScoreDataSet.where("yrpg_user = " + person.getPkValue()  + "and scienceProject = " + scoreCol.getString("scienceProject")); //科研项目进度表

                DynamicObject prizeDynamicObject = prizeEntry.addNew();//新增 业绩奖单据体
                prizeDynamicObject.set("yrpg_scienceproject", scoreCol.getString("scienceProject"));//科研项目
                prizeDynamicObject.set("yrpg_beforescore", scoreCol.getBigDecimal("lastProgress"));//上期比例
                prizeDynamicObject.set("yrpg_nowscore", scoreCol.getBigDecimal("nowProgress"));//本期比例
                prizeDynamicObject.set("yrpg_difference", scoreCol.getBigDecimal("realMoney"));//实发比例
                //调用业绩奖取数规则
                BigDecimal grantAmount = GrantCommonRules.grantAchievementRule(amount,scoreCol,allProjectScoreDataSet);
                prizeDynamicObject.set("yrpg_entryamount", grantAmount);//实发金额
                prizeDynamicObject.set("yrpg_issuecycle", scoreCol.getString("period"));//科研项目
            }
        }
    }

    /**
     * 判断本月是本季度的最后一月  达成奖
     */
    public boolean isLastMonth(DynamicObject period){
        boolean flag = false;
        //获取当前期间
        int currentMonth = period.getInt("periodnumber");
        //获取当前季度
        int currentQuarter = period.getInt("periodquarter");
        //本季度最后一月是本季度的3倍
        int lastMonthOfQuarter = currentQuarter * 3;
        if (currentMonth == lastMonthOfQuarter) {
            //本月是本季度的最后一个月
            flag =  true;
        }
        return flag;
    }


    /**
     * 获取本季度所有月份
     */
    public List<Integer> getAllMonth(DynamicObject period){
        int currentQuarter = period.getInt("periodquarter");//获取季度
        List<Integer> quarterMonths = new ArrayList<>();
        for (int monthNumber = (currentQuarter - 1) * 3 + 1; monthNumber <= currentQuarter * 3; monthNumber++) {
            quarterMonths.add(monthNumber);
        }
        return  quarterMonths;
    }
}