package yrpg.slm.pc.plugin.operate;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.PreparePropertysEventArgs;
import kd.bos.entity.plugin.args.EndOperationTransactionArgs;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import kd.sdk.plugin.Plugin;

import java.math.BigDecimal;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @author 白一帆
 * @desc 奖项计算明细审核时自动计算总金额并携带至源计算单工资汇总分录中。
 */
public class CalDetailOpPlugin extends AbstractOperationServicePlugIn implements Plugin {
    @Override
    public void onPreparePropertys(PreparePropertysEventArgs e) {
        super.onPreparePropertys(e);
        e.getFieldKeys().add("yrpg_business.number");
        e.getFieldKeys().add("srcid");
        e.getFieldKeys().add("yrpg_totalamount");
        e.getFieldKeys().add("yrpg_business.number");
        e.getFieldKeys().add("yrpg_business.number");
    }

    @Override
    public void endOperationTransaction(EndOperationTransactionArgs e) {
        super.endOperationTransaction(e);
        BigDecimal isReturn;
        if("audit".equals(e.getOperationKey())){
            isReturn = BigDecimal.ZERO;
        }else if("unaudit".equals(e.getOperationKey())){
            isReturn = new BigDecimal(-1);
        } else {
            return;
        }
        //当前奖项为计算奖项审核逻辑，增加判断是否为计算奖项信息
        boolean iscalcute = Arrays.stream(e.getDataEntities()).anyMatch(x -> {
            return "yrpg_wages_calate".equals(x.getString("yrpg_srcentity"));
        });
        if(iscalcute) {

            String billname = this.billEntityType.getName();
            //获取队伍信息，并获取唯一的奖项信息
            Map<String, List<DynamicObject>> yrpgBusinessNumber = Arrays.stream(e.getDataEntities())
                    .filter(x -> {
                        return x.getDynamicObject("yrpg_business") != null;
                    }).collect(Collectors.groupingBy(x -> {
                        return x.getDynamicObject("yrpg_business").getString("number");
                    }, Collectors.toList()));
            DynamicObject[] yrpgPcAwardTypes = BusinessDataServiceHelper.load("yrpg_pc_awardtype", "id", new QFilter[]{
                    new QFilter("status", QFilter.equals, "C")
                    , new QFilter("yrpg_compute", QFilter.equals, billname)
            });
            //根据奖项类型中的销售队伍进行分组。
            Map<String, DynamicObject> yrpgPcAwardTypesNumber = Arrays.stream(yrpgPcAwardTypes)
                    .filter(x->{
                        return x.getDynamicObject("yrpg_businessline")!=null;
                    })
                    .collect(Collectors.toMap(x -> {
                return x.getDynamicObject("yrpg_businessline").getString("number");
            }, y -> {
                return y;
            }));
            List<Long> srcid1 = Arrays.stream(yrpgPcAwardTypes).map(x -> {
                return x.getLong("srcid");
            }).collect(Collectors.toList());
            DynamicObject[] yrpgWagesCalates = BusinessDataServiceHelper.load("yrpg_wages_calate",
                    "id,yrpg_totalentryentity.yrpg_prize,yrpg_totalentryentity.yrpg_prizeprice1",
                    new QFilter[]{
                            new QFilter("id", QFilter.in, srcid1)
                    });
            //根据工资计算单的id进行分组
            Map<Long, DynamicObject> yrpgWagesCalatesid = Arrays.stream(yrpgWagesCalates).collect(Collectors.toMap(x -> {
                return x.getLong("id");
            }, y -> {
                return y;
            }));
            //根据计算奖项明细中的源单id分组。
            yrpgBusinessNumber.entrySet().forEach(x -> {
                Map<Long, DynamicObject> srcid2 = x.getValue().stream().collect(Collectors.toMap(x1 -> {
                    return x1.getLong("srcid");
                }, y -> {
                    return y;
                }));
                Set<Long> srcid = srcid2.keySet();

                //根据源单销售队伍字段取计算类型字段。
                DynamicObject yrpgPrize = yrpgPcAwardTypesNumber.get(x.getKey());
                srcid.forEach(x1 -> {
                    DynamicObject obj = yrpgWagesCalatesid.get(x1);
                    DynamicObjectCollection yrpgTotalentryentity = obj.getDynamicObjectCollection("yrpg_totalentryentity");
                    boolean b = yrpgTotalentryentity.stream().anyMatch(xx -> {
                        return xx.getDynamicObject("yrpg_prize").getString("yrpg_compute").equals(billname);
                    });
                    DynamicObject entry = null;
                    //工资计算单中已存在当前奖项分录，直接修改。不存在则新增
                    if (b) {
                        entry = yrpgTotalentryentity.stream().filter(x2 -> {
                            return x2.getDynamicObject("yrpg_prize").getString("yrpg_compute").equals(billname);
                        }).findFirst().orElse(null);
                    } else {
                        entry = yrpgTotalentryentity.addNew();
                        entry.set("yrpg_prize", yrpgPrize);
                    }
                    //覆盖计算单工资汇总金额字段
                    entry.set("yrpg_prizeprice1", srcid2.get(x1).getBigDecimal("yrpg_totalamount").multiply(isReturn));
                    BigDecimal yrpgTotalentryentity2 = obj.getDynamicObjectCollection("yrpg_totalentryentity").stream().map(x21 -> {
                        return x21.getBigDecimal("yrpg_prizeprice1");
                    }).reduce(BigDecimal::add).get();
                     obj.set("yrpg_totalamount",yrpgTotalentryentity2);
                });

            });
            SaveServiceHelper.save(yrpgWagesCalates);
        }
    }
}